% Modified by David Torun, for International Macroeconomics, FS 2021
%tot_cbcs.m -> own data
%country-by-country SVAR estimates and variance decompositions  in the chapter entitled ``Importable Goods, Exportable Goods  and the Terms of Trade'' of the book `Open Economy Macroeconomics,' by Mart?n Uribe and Stephanie Schmitt-Groh?, Princeton University Press 2017. 
%? Mart?n Uribe and Stephanie Schmitt-Groh?, 2016. 

clear all; clc;

%Set the size of the SVAR system 
sizevar = 5; %SVAR of the section entitled  `How Important Are Terms-of-Trade Shocks?'

%load the data (for regression analysis)
% Initialize variables.
filename = 'data_Q8.csv';
delimiter = ',';

formatSpec = '%f%f%f%f%f%f%[^\n\r]';
fileID = fopen(filename,'r');
dataArray = textscan(fileID, formatSpec, 'Delimiter', delimiter, 'TextType', 'string',  'ReturnOnError', false);
fclose(fileID);

datatable = table(dataArray{1:end-1}, 'VariableNames', {'year','tot','tb','y','c','i'});
clearvars filename delimiter formatSpec fileID dataArray ans;

T = max(datatable.year)-min(datatable.year)+1;
Nc = size(datatable,1)/(T);

% Get country labels
% Initialize variables.
filename = 'names_Q8.csv'; delimiter = {''};
formatSpec = '%s%[^\n\r]';
fileID = fopen(filename,'r');
dataArray = textscan(fileID, formatSpec, 'Delimiter', delimiter, 'TextType', 'string',  'ReturnOnError', false);
fclose(fileID);
labelsformatlab = table(dataArray{1:end-1}, 'VariableNames', {'labels'});
clearvars filename delimiter formatSpec fileID dataArray ans;

country = table2cell(labelsformatlab(:,:))'; %store country labels
clear labelsformatlab

% Get country iso codes --> importantly, sorted by country names to be
% consistent with labels above!
% Initialize variables.
filename = 'iso_Q8.csv'; delimiter = {''};
formatSpec = '%s%[^\n\r]';
fileID = fopen(filename,'r');
dataArray = textscan(fileID, formatSpec, 'Delimiter', delimiter, 'TextType', 'string',  'ReturnOnError', false);
fclose(fileID);
labelsformatlab = table(dataArray{1:end-1}, 'VariableNames', {'labels'});
clearvars filename delimiter formatSpec fileID dataArray ans;

country_iso = table2cell(labelsformatlab(:,:))'; %store country labels
clear labelsformatlab

% file name saved below
filename = ['tot_cbcs_own' num2str(sizevar)]; %this string variable is used to save the output of this program (see the last line of code).

%Retrieve dependent variables of interest -> can use variable names of
%table to easily access them!
toreg = table2array(datatable(:,{'tot', 'tb', 'y', 'c', 'i','year'}));

% Position of variables (columns in datatable)
tot_pos = 1;
tb_pos = 2;
y_pos = 3;
c_pos = 4;
i_pos = 5;
year_pos = 6;

% Need to log-quadratically detrend variables (tb is scaled by secular
% component of y and then quadratically detrended)
%which variables to log-quadratically detrend? -> use indices!
set_log = [tot_pos y_pos c_pos i_pos]; %columns of "toreg"

%which variables to quadratically detrend?
% set_lev = [ ];

%which variables to quadratically detrend after scaling by secular
%component?
set_sec = [tb_pos];

%%%Regressions
trendvar = datatable.year-min(datatable.year);

%%%log-quadratic detrending
for country_index = 1:Nc %detrending is computed separately by country!
    for col = set_log %detrend each variable separately --> set_log is the set of variables to be log-quadratically detrended (see above)
        rows = [1+(country_index-1).*T : (country_index).*T]; %rows for a specific country
        Y = log(toreg(rows,col)); %log of dependent variable!
        X = [ones(size(Y)) trendvar(rows,:) trendvar(rows,:).^2]; %constant, (year-1980), and (year-1980).^2
                %importantly, regress automatically takes out NaN values!!!
        results = ols(Y, X); %run regressions
        beta = results.beta; %store coefficients as variable beta  
        cyclical(rows,col) = (Y-X*beta) ; %store residuals (i.e., the cyclical component)
                   
    end
end

%%%quadratic detrending scaled by secular component of y
y_sec = exp(log(toreg(:,y_pos))-cyclical(:,y_pos)); 
            %store secular component of y (or, exponential of fitted value);

for country_index = 1:Nc %same as above!
    for col = set_sec %detrend each variable separately --> set_sec is the set of variables to be quadratically detrended and scaled by y_sec (cf. above)
        rows = [1+(country_index-1).*T : (country_index).*T]; %same as above
        Y = toreg(rows,col)./y_sec(rows,:); %level of dependent variable scaled by y_sec!
        X = [ones(size(Y)) trendvar(rows,:) trendvar(rows,:).^2]; %same as above
        results = ols(Y, X); %run regressions
        beta = results.beta; %store coefficients as variable beta  
        cyclical(rows,col) = (Y-X*beta) ; %store residuals
    end
end
    clear rows Y X beta; %not needed anymore
    
take_out = zeros(Nc,1);    
for country_index = 1:Nc
    if isreal(cyclical([1+(country_index-1).*T : (country_index).*T]))~=1;
        take_out(Nc) = 1;
    end
end

for c = 1:Nc
    tot_data(:,:,c) = [toreg(1+(c-1)*T:c*T,year_pos) cyclical(1+(c-1)*T:c*T,:)];
    tot_data_cell{1,c} = table2cell(array2table([toreg(1+(c-1)*T:c*T,year_pos) cyclical(1+(c-1)*T:c*T,:)]));
end

if sizevar==5;
nvar = 2:6; %tot, tb, gdp, c, i, in this order
end
if sizevar==2
nvar = 2:3 %tot, tb, in this order
end

nv = length(nvar);
ncou = length(tot_data);
A = zeros(nv,nv,ncou);
A0 = eye(nv,nv);
A0 = repmat(A0,[1 1 ncou]);
PI = zeros(nv,nv,ncou);
PI1 = zeros(nv,nv,ncou);
hx = zeros(nv,nv,ncou);
sigx = zeros(nv,nv,ncou);
ssigx = zeros(nv,nv,ncou);
v_share = zeros(nv,ncou);
R2 = zeros(nv,ncou);

for k=1:ncou
d = tot_data(:,nvar,k);
d = lagg(d);

Y = d(:,1:end/2);
w = d(:,end/2+1:end);
X = [w w(:,1)*0+1];
T = size(X,1); %total number of observations

%the system is x_t = Ax_{t-1} + u_t
u = zeros(T,nv); %regression errors
%perform regressions
%TOT regression
y = Y(:,1);
w = [X(:,1) X(:,end)];
aa = w\y;
A(1,1,k) = aa(1);
u(:,1) =  y - w*aa;

%Estimate remaining equations
aa = [y X]\Y(:,2:end);
A(2:end,:,k) = aa(2:nv+1,:)';
A0(2:end,1,k) = -aa(1,:)';
u(:,2:end) = Y(:,2:end)-[y X]*aa;
R2(1:nv,k) = (1-var(u)./var(Y))';
SIGMA  = cov(u);
PI1(:,:,k) =chol(SIGMA,'lower');
PI(:,:,k) = A0(:,:,k)\PI1(:,:,k);
hx(:,:,k) = A0(:,:,k)\A(:,:,k);

[~,sigx(:,:,k)]=mom(eye(nv),hx(:,:,k),PI(:,:,k)*PI(:,:,k)');

PI1 = PI(:,:,k);
PI1(:,2:end) = 0;
[~,ssigx(:,:,k)]=mom(eye(nv),hx(:,:,k),PI1*PI1');

[~,vdx]=variance_decomposition(eye(nv),hx(:,:,k),PI(:,:,k));

v_share(:,k) = vdx(1,:)*100;

var_svar_tot(:,k) = diag(ssigx(:,:,k)); %variances conditional on tot shocks implied by empirical SVAR model
var_svar(:,k) = diag(sigx(:,:,k)); %total variances implied by empirical SVAR model

%var_share(k, 1:nv) =  (diag(ssigx(:,:,k))./ diag(sigx(:,:,k)));
end

var_svar_tot_ratio_cbc = var_svar_tot./var_svar; 

eval(['save ' filename '.mat filename hx PI R2 ncou country country_iso nv ssigx v_share var_svar var_svar_tot var_svar_tot_ratio_cbc'])